# Generated by roxygen2: do not edit by hand

export(Contrast)
export(Counts)
export(DBAmmd)
export(Dists)
export(Genome)
export(Hists)
export(Reads)
export(Regions)
export(Samples)
export(compDists)
export(compHists)
export(compPvals)
export(estimateFragmentCenters)
export(getPeakReads)
export(metaData)
export(numPeaks)
export(numSamples)
export(plotDISTS4Peak)
export(plotDists)
export(plotPeak)
export(reportResults)
export(runShinyMMDiff2)
export(setContrast)
export(setRegions)
exportClasses(DBAmmd)
import(BSgenome)
import(Biobase)
import(Biostrings)
import(GenomicRanges)
import(RColorBrewer)
import(Rsamtools)
import(S4Vectors)
import(ggplot2)
import(grDevices)
import(graphics)
import(locfit)
import(parallel)
import(shiny)
importFrom(stats,ccf)
importFrom(stats,ks.test)
importFrom(stats,median)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,predict)
importFrom(stats,pt)
importFrom(stats,quantile)
importFrom(stats,residuals)
importFrom(utils,read.csv)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
